DRIVER_CONFIG = {
    'google': {
        'access_token_url': 'https://www.googleapis.com/oauth2/v4/token',
        'authorization_url': 'https://accounts.google.com/o/oauth2/v2/auth',
        'profile_url': 'https://www.googleapis.com/oauth2/v2/userinfo'
    },
    'github': {
        'access_token_url': 'https://github.com/login/oauth/access_token',
        'authorization_url': 'http://github.com/login/oauth/authorize',
        'profile_url': 'https://api.github.com/user'
    }
}



""" The Google Driver Module """
import json
import urllib

import requests

from authicola.settings import DRIVER_CONFIG


class GoogleDriver:
    access_token_url = DRIVER_CONFIG['google']['access_token_url']
    authorization_url = DRIVER_CONFIG['google']['authorization_url']
    profile_url = DRIVER_CONFIG['google']['profile_url']
    """
    Required: config
    Accepts: scopes, state
    """
    def __init__(self, config, **kwargs):
        # optionally specify scopes, otherwise default to Authicola config
        self._scopes = kwargs.get('scopes', config.get('scopes'))
        self._state = kwargs.get('state')
        self._config = config

    def redirect_uri(self):
        """ Generate the redirect_uri to Google for client authorization """
        if self._scopes:
            scopes = ' '.join(self._scopes)
        else:
            scopes = ''
        try:
            params = dict(
                scope=scopes,
                access_type='offline',
                redirect_uri=self._config['redirect_uri'],
                response_type='code',
                client_id=self._config['client_id']
            )
            if self._state:
                params.update(state=self._state)
            endpoint = \
                '{auth_url}?{params}' \
                .format(
                    auth_url=self.authorization_url,
                    params=urllib.parse.urlencode(params)
                )
        except KeyError as exc:
            raise KeyError(
                'Social auth config for {name} is missing a required key: '
                '{err}'.format(name=__name__, err=str(exc))
            )
        except Exception as exc:
            raise Exception(
                'Error creating redirect_url: {err}'
                .format(err=str(exc))
            )

        return endpoint

    def scopes(self, *args):
        """ Set Scopes """
        self._scopes = args
        return self

    def state(self, state):
        """ Set state """
        if not isinstance(state, str):
            raise Exception('state param must be a string')
        self._state = state
        return self

    """
    Required: params
    Accepts: state
    """
    def user(self, params, state=None):
        """ Try and get the user from the oAuth flow """
        code = params.get('code')
        err = params.get('error')
        state_param = params.get('state')

        # catch returned error param on callback
        if not code and err:
            raise Exception(
                'Authorization failed with error: {err}'
                .format(err=err)
            )
        # catch invalid params argument
        elif not code:
            raise Exception(
                'Authorization code is missing from params: {params}'
                .format(params=str(params))
            )

        if state and state != state_param:
            raise Exception(
                'Invalid state parameter detected'
            )

        try:
            payload = dict(
                code=code,
                grant_type='authorization_code',
                redirect_uri=self._config['redirect_uri'],
                client_id=self._config['client_id'],
                client_secret=self._config['client_secret'],
            )
        except KeyError as exc:
            raise KeyError(
                'Social auth config for {name} is missing a required key: '
                '{err}'.format(name=__name__, err=str(exc))
            )
        except Exception as exc:
            raise Exception(
                'Error creating request body: {err}'
                .format(err=str(exc))
            )

        # Request an access token from the code in the request
        headers = {
            'Accept': 'application/json'
        }
        res = requests.post(
            self.access_token_url,
            headers=headers,
            data=payload
        )
        res.raise_for_status()

        # Get the access token from the response
        try:
            token_response = res.json()
        except ValueError as exc:
            raise ValueError(
                'Error reading token response: {err}'
                .format(str(exc))
            )

        # Exchange the access token for a user
        headers = {
            'Authorization': 'Bearer {}'
            .format(token_response.get('access_token'))
        }

        res = requests.get(self.profile_url, headers=headers)
        res.raise_for_status()

        try:
            return res.json()
        except ValueError as exc:
            raise ValueError(
                'Error reading user response: {err}'
                .format(str(exc))
            )



""" Authicola root class """
# from config import authicola
from authicola.drivers import (
    GoogleDriver,
    GitHubDriver
)


class Authicola:
    """
    Required: config
    """
    def __init__(self, config):
        self.config = config

    """
    Required: name
    Accepts: scopes
    """
    def driver(self, name, **kwargs):
        try:
            chosen_driver = None
            if name == 'google':
                chosen_driver = GoogleDriver
            elif name == 'github':
                chosen_driver = GitHubDriver
            if chosen_driver:
                return chosen_driver(
                    config=self.config[name],
                    **kwargs
                )
        except KeyError:
            raise KeyError(
                'Config for driver "{driver}" not found'
                .format(driver=name)
            )
        except Exception as exc:
            raise KeyError(
                'Error loading driver "{driver}": {err}'
                .format(driver=name, err=str(exc))
            )
        
        raise NotImplementedError(
            'Social driver "{driver}" not found'
            .format(driver=name)
        )



"""Metadata for Authicola package."""
__title__ = 'authicola'
__description__ = 'A convenience libary of OAuth2.0 providers'
__url__ = 'https://github.com/garyburgmann/authicola'
__version__ = '0.0.6'
__author__ = 'Gary Burgmann'
__author_email__ = 'garyburgmann@gmail.com'
__license__ = 'MIT'

from .Authicola import Authicola



from setuptools import setup
from os import path

import authicola as meta

this_directory = path.abspath(path.dirname(__file__))
with open(path.join(this_directory, 'README.md'), encoding='utf-8') as f:
    long_description = f.read()


setup(
    name=meta.__name__,
    version='0.0.6',
    packages=[
        'authicola',
        'authicola.drivers',
    ],
    install_requires=[
        'requests'
    ],
    include_package_data=True,
    description=meta.__description__,
    long_description=long_description,
    long_description_content_type='text/markdown',
    author=meta.__author__,
    author_email=meta.__author_email__,
    url=meta.__url__,
    license=meta.__license__,
    python_requires=">=3.4",
    keywords=['oauth', 'oauth2.0', 'authentication', 'python3', 'google'],
)
